// -*- c++ -*-
// Generated by gmmproc 2.46.1 -- DO NOT MODIFY!
#ifndef _LIBGDAMM_NUMERIC_H
#define _LIBGDAMM_NUMERIC_H


#include <glibmm/ustring.h>
#include <sigc++/sigc++.h>

/* Copyright 2011 libgdamm Development Team
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or(at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <glibmm/ustring.h>
#include <glibmm/value.h>


#ifndef DOXYGEN_SHOULD_SKIP_THIS
extern "C" { typedef struct _GdaNumeric GdaNumeric; }
#endif

namespace Gnome
{

namespace Gda
{


/** TODO
 */
class Numeric
{
  public:
#ifndef DOXYGEN_SHOULD_SKIP_THIS
  typedef Numeric CppObjectType;
  typedef GdaNumeric BaseObjectType;
#endif /* DOXYGEN_SHOULD_SKIP_THIS */

  /** Get the GType for this class, for use with the underlying GObject type system.
   */
  static GType get_type() G_GNUC_CONST;

  Numeric();

  explicit Numeric(GdaNumeric* gobject, bool make_a_copy = true);

  Numeric(const Numeric& other);
  Numeric& operator=(const Numeric& other);

  Numeric(Numeric&& other) noexcept;
  Numeric& operator=(Numeric&& other) noexcept;

  ~Numeric() noexcept;

  void swap(Numeric& other) noexcept;

  ///Provides access to the underlying C instance.
  GdaNumeric*       gobj()       { return gobject_; }

  ///Provides access to the underlying C instance.
  const GdaNumeric* gobj() const { return gobject_; }

  ///Provides access to the underlying C instance. The caller is responsible for freeing it. Use when directly setting fields in structs.
  GdaNumeric* gobj_copy() const;

protected:
  GdaNumeric* gobject_;

private:

  
public:

  
  void set_from_string(const Glib::ustring& str);
  
  void set_double(double number);
  
  double get_double() const;
  
  void set_precision(long precision);
  
  long get_precision() const;
  
  void set_width(long width);
  
  long get_width();
  
  Glib::ustring get_string() const;


};


} // namespace Gda
} // namespace Gnome


namespace Gnome
{

namespace Gda
{

/** @relates Gnome::Gda::Numeric
 * @param lhs The left-hand side
 * @param rhs The right-hand side
 */
inline void swap(Numeric& lhs, Numeric& rhs) noexcept
  { lhs.swap(rhs); }

} // namespace Gda

} // namespace Gnome

namespace Glib
{

/** A Glib::wrap() method for this object.
 * 
 * @param object The C instance.
 * @param take_copy False if the result should take ownership of the C instance. True if it should take a new copy or ref.
 * @result A C++ instance that wraps this C instance.
 *
 * @relates Gnome::Gda::Numeric
 */
Gnome::Gda::Numeric wrap(GdaNumeric* object, bool take_copy = false);

#ifndef DOXYGEN_SHOULD_SKIP_THIS
template <>
class Value<Gnome::Gda::Numeric> : public Glib::Value_Boxed<Gnome::Gda::Numeric>
{};
#endif /* DOXYGEN_SHOULD_SKIP_THIS */

} // namespace Glib


#endif /* _LIBGDAMM_NUMERIC_H */

