// -*- c++ -*-
// Generated by gmmproc 2.46.1 -- DO NOT MODIFY!
#ifndef _LIBGDAMM_DATAMODELITER_P_H
#define _LIBGDAMM_DATAMODELITER_P_H


#include <libgdamm/private/set_p.h>

#include <glibmm/class.h>

namespace Gnome
{

namespace Gda
{

class DataModelIter_Class : public Glib::Class
{
public:
#ifndef DOXYGEN_SHOULD_SKIP_THIS
  typedef DataModelIter CppObjectType;
  typedef GdaDataModelIter BaseObjectType;
  typedef GdaDataModelIterClass BaseClassType;
  typedef Set_Class CppClassParent;
  typedef GdaSetClass BaseClassParent;

  friend class DataModelIter;
#endif /* DOXYGEN_SHOULD_SKIP_THIS */

  const Glib::Class& init();


  static void class_init_function(void* g_class, void* class_data);

  static Glib::ObjectBase* wrap_new(GObject*);

protected:

  //Callbacks (default signal handlers):
  //These will call the *_impl member methods, which will then call the existing default signal callbacks, if any.
  //You could prevent the original default signal handlers being called by overriding the *_impl method.
  static void end_of_data_callback(GdaDataModelIter* self);
  static void row_changed_callback(GdaDataModelIter* self, gint p0);

  //Callbacks (virtual functions):
};


} // namespace Gda

} // namespace Gnome


#endif /* _LIBGDAMM_DATAMODELITER_P_H */

