// -*- c++ -*-
// Generated by gmmproc 2.46.1 -- DO NOT MODIFY!
#ifndef _LIBGDAMM_HANDLERTIME_H
#define _LIBGDAMM_HANDLERTIME_H


#include <glibmm/ustring.h>
#include <sigc++/sigc++.h>

// -*- C++ -*- //

/* handlertime.h
 *
 * Copyright 2006 libgdamm Development Team
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or(at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <glibmm/object.h>
#include <libgdamm/datahandler.h>
#include <libgda/handlers/gda-handler-time.h>


#ifndef DOXYGEN_SHOULD_SKIP_THIS
typedef struct _GdaHandlerTime GdaHandlerTime;
typedef struct _GdaHandlerTimeClass GdaHandlerTimeClass;
#endif /* DOXYGEN_SHOULD_SKIP_THIS */


#ifndef DOXYGEN_SHOULD_SKIP_THIS
namespace Gnome
{

namespace Gda
{ class HandlerTime_Class; } // namespace Gda

} // namespace Gnome
#endif //DOXYGEN_SHOULD_SKIP_THIS

namespace Gnome
{

namespace Gda
{

/** Default handler for time, date and timestamp.
 *
 * @ingroup DataHanders
 */

class HandlerTime : public Glib::Object,
                    public DataHandler
{
  
#ifndef DOXYGEN_SHOULD_SKIP_THIS

public:
  typedef HandlerTime CppObjectType;
  typedef HandlerTime_Class CppClassType;
  typedef GdaHandlerTime BaseObjectType;
  typedef GdaHandlerTimeClass BaseClassType;

  // noncopyable
  HandlerTime(const HandlerTime&) = delete;
  HandlerTime& operator=(const HandlerTime&) = delete;

private:  friend class HandlerTime_Class;
  static CppClassType handlertime_class_;

protected:
  explicit HandlerTime(const Glib::ConstructParams& construct_params);
  explicit HandlerTime(GdaHandlerTime* castitem);

#endif /* DOXYGEN_SHOULD_SKIP_THIS */

public:

  HandlerTime(HandlerTime&& src) noexcept;
  HandlerTime& operator=(HandlerTime&& src) noexcept;

  virtual ~HandlerTime() noexcept;

  /** Get the GType for this class, for use with the underlying GObject type system.
   */
  static GType get_type()      G_GNUC_CONST;

#ifndef DOXYGEN_SHOULD_SKIP_THIS


  static GType get_base_type() G_GNUC_CONST;
#endif

  ///Provides access to the underlying C GObject.
  GdaHandlerTime*       gobj()       { return reinterpret_cast<GdaHandlerTime*>(gobject_); }

  ///Provides access to the underlying C GObject.
  const GdaHandlerTime* gobj() const { return reinterpret_cast<GdaHandlerTime*>(gobject_); }

  ///Provides access to the underlying C instance. The caller is responsible for unrefing it. Use when directly setting fields in structs.
  GdaHandlerTime* gobj_copy();

private:

  
protected:

  // TODO: Note that this wraps only the no_locale version of GdaHandlerTime. The
  // other version calls some private function that we have no access to. This
  // should be fixed in libgnomedb, I think. armin.
  HandlerTime();

public:
  
  static Glib::RefPtr<HandlerTime> create();


public:

public:
  //C++ methods used to invoke GTK+ virtual functions:

protected:
  //GTK+ Virtual Functions (override these to change behaviour):

  //Default Signal Handlers::


};

} // namespace Gda
} // namespace Gnome


namespace Glib
{
  /** A Glib::wrap() method for this object.
   * 
   * @param object The C instance.
   * @param take_copy False if the result should take ownership of the C instance. True if it should take a new copy or ref.
   * @result A C++ instance that wraps this C instance.
   *
   * @relates Gnome::Gda::HandlerTime
   */
  Glib::RefPtr<Gnome::Gda::HandlerTime> wrap(GdaHandlerTime* object, bool take_copy = false);
}


#endif /* _LIBGDAMM_HANDLERTIME_H */

